package scacchi.main;

import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;
import scacchi.main.EccezioniPartita.*;

/**
 * Classe per la memorizzazione e l'esecuzione dei comandi per il
 * gioco degli scacchi. <br>
 * La classe &egrave; astratta perch&egrave; i metodi {@link #nuovaPartita()}
 * e {@link #nuovaPartita(int, ArgomentiComando.ListaPosPezziTesto)} 
 * sono astratti. In una classe concreta che estende questa class astratta
 * i due metodi suddetti ritorneranno due nuove istanze della implementazione 
 * della classe {@link Partita} che si sta usando (ad esempio, chiamando i
 * corrispondenti costruttori, come in {@link ComandoScacchi}).
 * <br>
 * Comandi validi
 * <table>
 * <tr><td>  #  </td>
 *         <td>  chiede il numero della mossa  </td></tr>
 * <tr><td>  @  </td>
 *         <td>  chiede quale giocatore ha il tratto  </td></tr>
 * <tr><td>  e <em>mossa</em>  </td>
 *         <td>  esegue <em>mossa</em>  </td></tr>
 * <tr><td>  l    </td>
 *         <td>  lista tutte le mosse della partita  </td></tr>
 * <tr><td>  l <em>n</em>  </td>
 *         <td>  lista le ultime <em>n</em> mosse della partita  </td></tr>
 * <tr><td>  i  </td>
 *         <td>  torna indietro di una semi-mossa  </td></tr>
 * <tr><td>  i <em>n</em>  </td>
 *         <td>  torna indietro di <em>n</em> semi-mosse  </td></tr>
 * <tr><td>  p  </td>
 *         <td>  posizione dei pezzi  </td></tr>
 * <tr><td>  c <em>casa</em>  </td>
 *         <td>  case controllate dal pezzo in <em>casa</em>  </td></tr>
 * <tr><td>  m  </td>
 *         <td>  mosse valide per il giocatore che ha il tratto
 * <tr><td>  m <em>casa</em>  </td>
 *         <td>  mosse valide per il pezzo in <casa>  </td></tr>
 * <tr><td>  n  </td>
 *         <td>  ricomincia una nuova partita  </td></tr>
 * <tr><td>  n <em>i</em> <em>lista-pos-pezzi</em>  </td>
 *         <td>  ricomincia una nuova partita a partire dalla
 *               posizione fornita in <em>lista-pos-pezzi</em>,
 *               <em>i</em> indica il numero della mossa, se positivo il
 *               tratto &egrave; al BIANCO, se negativo &egrave; al NERO.
 *               </td></tr>
 * </table>
 * @see ComandoScacchi
 * @author Stefano Guerrini
 * @version 04/12/07
 */
public abstract class Comando {
    // ----------------------------------------------------------------------
    // --                              static                              --
    // ----------------------------------------------------------------------

    // ----  constants  ----

    /** Comando numero mosse */
    public static final char COM_NUM_MOSSA = '#';
    /** Comando giocatore che ha la mossa */
    public static final char COM_GIOC_MUOVE = '@';
    /** Comando esegui mossa */
    public static final char COM_ESEGUI_MOSSA = 'e';
    /** Comando lista mosse */
    public static final char COM_LISTA_MOSSE = 'l';
    /** Comando indietro */
    public static final char COM_INDIETRO = 'i';
    /** Comando lista posizione pezzi */
    public static final char COM_POSIZIONE = 'p';
    /** Comando case controllate */
    public static final char COM_CASE_CONTR = 'c';
    /** Comando mosse valide */
    public static final char COM_MOSSE_VALIDE = 'm';
    /** Comandi per partita in corso che possono avere zero argomenti */
    private static final char COM_NO_ARG[] = {
	COM_NUM_MOSSA,
	COM_GIOC_MUOVE,
	COM_LISTA_MOSSE,
	COM_INDIETRO,
	COM_POSIZIONE,
	COM_MOSSE_VALIDE
    };
    /** Comandi per partita in corso che possono avere un numero come arg. */
    private static final char COM_ARG_NUM[] = {
	COM_LISTA_MOSSE,
	COM_INDIETRO
    };
    /** Comandi per partita in corso che possono avere una casa come arg. */
    private static final char COM_ARG_CASA[] = {
	COM_CASE_CONTR,
	COM_MOSSE_VALIDE
    };
    /** Comandi per partita in corso che possono avere una mossa come arg. */
    private static final char COM_ARG_MOSSA[] = {
	COM_ESEGUI_MOSSA
    };
    /** Comando nuova partita */
    public static final char COM_NUOVA_PARTITA = 'n';

    // ----  static methods  ----
    
    /**
     * Cerca la prima posizione di un carattere in un array.
     * @param ch Il carattere da cercare.
     * @param v  L'array in cui cercare.
     * @return La prima posizione in cui si trova il carattere o -1
     *         se il carattere non appare nell'array.
     */
    static int posCharInArray(char ch, char v[]) {
	for(int i = 0; i < v.length; i++)
	    if (ch == v[i])
		return i;
	return -1;
    }

    /**
     * Verifica se un carattere &egrave; contenuto in un array.
     * @param ch Il carattere da cercare.
     * @param v  L'array in cui cercare.
     * @return true se il carattere appare nella stringa, false altrimenti.
     */
    static boolean charInArray(char ch, char v[]) {
	return (posCharInArray(ch, v) >= 0) ? true : false;
    }

    // ----------------------------------------------------------------------
    // --                             instance                             --
    // ----------------------------------------------------------------------

    // ----  instance fields  ----

    /** Comando da eseguire. */
    private final char ch;
    /** Lista degli argomenti del comando. */
    private final ArgomentiTesto args;

    // ----  instance classes ----

    /** 
     * Classe per la memorizzazione e varifica delle stringhe degli
     * argomenti di un comando.
     */
    class ArgomentiTesto {
	/** Codice per comando senza argomenti */
	private static final byte NO_ARG = 0;
	/** Codice per comando con argomento numerico */
	private static final byte ARG_NUM = 1;
	/** Codice per comando con argomento casa */
	private static final byte ARG_CASA = 2;
	/** Codice per comando con argomento mossa */
	private static final byte ARG_MOSSA = 3;
	/** Codice per comando con argomento posizione iniziale */
	private static final byte ARG_POS_INIZ = 4;

	/** Memorizza il codice della lista di argomenti del comando */
	private byte tag = 0;
	/** Memorizza un valore numerico o intero */
	private int n = 0;
	/** Memorizza una casa */
	private CasaTesto c = null;
	/** Memorizza una mossa */
	private MossaTesto m = null;
	/** Memorizza una lista di posizioni di pezzi */
	private ListaPosPezziTesto l = null;

	/**
	 * Costruisce la rappresentazione per gli argomenti di un comando
	 * dopo aver verificato che la lista &egrave; corretta per
	 * il comando in esame.
	 * @param s  La stringa degli argomenti.
	 * @param ch Il carattere corrispondente al comando in esame.
	 */
	ArgomentiTesto(String s) throws NotazioneException {
	    if (Comando.charInArray(ch, COM_NO_ARG) && s.length() == 0) {
		tag = NO_ARG;  // comando senza argomenti
	    } else if (Comando.charInArray(ch, COM_ARG_NUM)) {
		try {
		    tag = ARG_NUM; // comando con un argomento numerico
		    n = Integer.parseInt(s);
		    if (n < 0) // val. num. non valido
			throw new ValNumericoException(s);
		} catch (NumberFormatException e) {
		    throw new ValNumericoException(s); // val. num. non valido
		}
	    } else if (Comando.charInArray(ch, COM_ARG_CASA)) {
		tag = ARG_CASA; // comando con una casa come argomento
		c = new CasaTesto(s);
	    } else if (Comando.charInArray(ch, COM_ARG_MOSSA)) {
		tag = ARG_MOSSA; // comando con una mossa come argomento
		m = new MossaTesto(s);
	    } else if (ch == COM_NUOVA_PARTITA) {
		tag = ARG_POS_INIZ;
		if (s.length() == 0) {
		    // nessun argomento: comincia dalla pos. iniziale del gioco
		    tag = NO_ARG;
		} else {
		    // dividi la stringa degli argomenti in due parti:
		    // il valore intero e la lista di posizioni di pezzi
		    String v[] = s.split("\\s+", 2);
		    // la prima stringa deve essere un intero
		    try {
			n = Integer.parseInt(v[0]);
		    } catch (NumberFormatException e) {
			throw new ValInteroException(v[0]);
		    }
		    l = new ListaPosPezziTesto(v[1]);
		}
 	    } else
		throw new ArgomentiException(s);
	}

	/**
	 * Indica se un comando &egrave; senza argomenti.
	 * @return true se il comando non ha una lista di argomenti
	 *         associata, false altrimenti.
	 */
	boolean noArg() { return tag == NO_ARG;	}

	/**
	 * Ritorna l'argomento casa del comando.
	 * @return La casa memorizzata nell'argomento se si tratta
	 *         di un comando che ha una casa come argomento.
	 * @throws java.lang.Error()
	 *         se il comando che ha una casa come argomento.
	 */
	CasaTesto casa() {
	    if (tag != ARG_CASA)
		throw new Error();
	    return c;
	}

	/**
	 * Ritorna l'argomento mossa del comando.
	 * @return La mossa memorizzata nell'argomento se si tratta
	 *         di un comando che ha una mossa come argomento.
	 * @throws java.lang.Error()
	 *         se il comando che ha una mossa come argomento.
	 */
	MossaTesto mossa() {
	    if (tag != ARG_MOSSA)
		throw new Error();
	    return m;
	}

	/**
	 * Ritorna l'argomento numerico del comando.
	 * @return Il valore numerico memorizzato nell'argomento se si tratta
	 *         di un comando che ha un valore numerico come argomento.
	 * @throws java.lang.Error()
	 *         se il comando che ha un valore numerico come argomento.
	 */
	int valNum() {
	    if (tag != ARG_NUM)
		throw new Error();
	    return n;
	}

	/**
	 * Ritorna l'argomento intero del comando.
	 * @return L'intero memorizzato nell'argomento se si tratta
	 *         di un comando che ha un intero come argomento.
	 * @throws java.lang.Error()
	 *         se il comando che ha un intero come argomento.
	 */
	int valInt() {
	    if (tag != ARG_POS_INIZ)
		throw new Error();
	    return n;
	}

	/**
	 * Ritorna la lista di posizioni di pezzi del comando.
	 * @return La lista di posizioni di pezzi nell'argomento se si tratta
	 *         di un comando che ha una lista di posizioni di pezzi
	 *         come argomento.
	 * @throws java.lang.Error()
	 *         se il comando che ha una lista di posizioni di pezzi
	 *         come argomento.
	 */
	ListaPosPezziTesto listaPosPezzi() {
	    if (tag != ARG_POS_INIZ)
		throw new Error();
	    return l;
	}

	/**
	 * Ricostruisce la stringa del comando.
	 */
	public String toString() {
	    String s = "";
	    // ricostuisce la stringa degli argomenti
	    switch (tag) {
	    case NO_ARG:
		break;
	    case ARG_NUM:
		s = "" + n;
		break;
	    case ARG_CASA:
		s = c.toString();
		break;
	    case ARG_MOSSA:
		s = m.toString();
		break;
	    case ARG_POS_INIZ:
		System.out.println(l.toString());
		s = "" + n + " " + l.toString();
		break;
	    default:
		throw new Error();
	    }
	    // appende gli argomenti al carattere del comando
	    return s;
	}
    }

    // ----  constructors  ----

    /**
     * Decompone e verifica una stringa con un comando.
     * @param s La stringa del comando: la stringa deve iniziare con
     *          il carattere del comando (eventuali spazi prima del
     *          comando deve essere eliminati prima di chiamare la funzione).
     */
    protected Comando(String s) throws NotazioneException {
	ch = s.charAt(0);
	args = new ArgomentiTesto(s.substring(1).trim());
	// System.out.println(":" + ch + ":" + args.toString() + ":");
    }

    // ----  instance methods  ----

    /**
     * Crea una nuova partita che parte dalla posizione iniziale del gioco. 
     * @return Una nuova istanza di una classe che implementa {@link Partita}.
     */
    public abstract Partita nuovaPartita();

    /**
     * Crea una nuova partita che parte dalla posizione passata negli
     * argomenti.
     * @param n La mossa da cui parte la partita. Un valore positivo indica
     *          che il tratto &egrave; al BIANCO, un valore negativo che il
     *          tratto &egrave; al NERO.
     * @param l La posizione dei pezzi da cui comincia la partita.
     * @return Una nuova istanza di una classe che implementa {@link Partita}.
     * @throws scacchi.amin.EccezioniPartita.PosizioneInizialeException
     *         Se la posizione iniziale non &egrave; valida. 
     */
    public abstract Partita nuovaPartita(int n, ListaPosPezziTesto l)
	throws PosizioneInizialeException;

    /**
     * Verifica se si tratta di un comando che inizia una nuova partita.
     * @return true se il comando inizia una nuova partita, false altrimenti.
     */
    public boolean isComandoNuovaPartita() { return ch == COM_NUOVA_PARTITA; }

    /**
     * Esegue un comando di nuova partita.
     * @return Una nuova partita.
     * @throws Partita.PartitaException segnala un eventuale errore
     *        nell'esecuzione del comando.
     */
    public Partita eseguiComandoNuovaPartita()
	throws PosizioneInizialeException {
	Partita p;
	if (ch != COM_NUOVA_PARTITA)
	    throw new Error();
	if (args.noArg())
	    p = nuovaPartita();
	else
	    p = nuovaPartita(args.valInt(), args.listaPosPezzi());
	return p;
    }

    /**
     * Esegue un comando su di una certa partita.
     * @param partita Il comando da eseguire.
     * @return L'output del comando.
     * @throws Partita.PartitaException segnala un eventuale errore
     *         nell'esecuzione del comando.
     */
    public String eseguiComandoPartita(Partita partita)
	throws NotazioneException, PartitaException {
	String out = ""; // output del comando

	switch (ch) {
	case COM_NUM_MOSSA:
	    int i = partita.getNumMossa();
	    out = "" + (i > 0 ? i : -i);
	    break;
	case COM_GIOC_MUOVE:
	    out = partita.getNumMossa() > 0 ? "BIANCO" : "NERO";
	    break;
	case COM_ESEGUI_MOSSA:
	    out = partita.eseguiMossa(args.mossa()).toString();
	    break;
	case COM_LISTA_MOSSE:
	    if (args.noArg())
		out = partita.listaMosse().toString();
	    else
		out = partita.listaMosse(args.valNum()).toString();
	    break;
	case COM_INDIETRO:
	    if (args.noArg())
		out = partita.tornaIndietro().toString();
	    else
		out = partita.tornaIndietro(args.valNum()).toString();
	    break;
	case COM_POSIZIONE:
	    out = partita.getPosizionePezzi().toString();
	    break;
	case COM_CASE_CONTR:
	    out = partita.caseControllate(args.casa()).toString();
	    break;
	case COM_MOSSE_VALIDE:
	    if (args.noArg())
		out = partita.getMosseValide().toString();
	    else
		out = partita.getMosseValide(args.casa()).toString();
	    break;
	default:
	    throw new Error();
	}
	return out;
    }

    /** Ricostruisce la stringa del comando. */
    public String toString() { return "" + ch + " " + args.toString(); }
}
